@@ -14,6 +14,7 @@ use cargo::util::command_prelude::{ArgMatches, ArgMatchesExt, CompileMode, Profi
 use cargo::util::interning::InternedString;
 use cargo::{CliResult, Config};
 
+use anyhow::Context as _;
 use cargo_util::paths::{copy, create, create_dir_all, open, read, read_bytes, write};
 use semver::Version;
 
@@ -43,7 +44,7 @@ fn build_include_file(
     let warning = config.autogen_warning.unwrap_or_default();
     let version_info = format!(
         "\n#define {0}_MAJOR {1}\n#define {0}_MINOR {2}\n#define {0}_PATCH {3}\n",
-        name.to_uppercase(),
+        name.to_uppercase().replace('-', "_"),
         version.major,
         version.minor,
         version.patch
@@ -111,7 +112,7 @@ fn patch_target(
     let manifest = pkg.manifest_mut();
     let targets = manifest.targets_mut();
 
-    let mut kinds: Vec<_> = libkinds
+    let kinds: Vec<_> = libkinds
         .iter()
         .map(|&kind| match kind {
             "staticlib" => CrateType::Staticlib,
@@ -120,8 +121,6 @@ fn patch_target(
         })
         .collect();
 
-    kinds.push(CrateType::Lib);
-
     for target in targets.iter_mut() {
         if target.is_lib() {
             target.set_kind(TargetKind::Lib(kinds.clone()));
@@ -402,15 +401,43 @@ pub struct PkgConfigCApiConfig {
     pub strip_include_path_components: usize,
 }
 
+#[derive(Debug)]
+pub enum VersionSuffix {
+    Major,
+    MajorMinor,
+    MajorMinorPatch,
+}
+
 #[derive(Debug)]
 pub struct LibraryCApiConfig {
     pub name: String,
     pub version: Version,
     pub install_subdir: Option<String>,
     pub versioning: bool,
+    pub version_suffix_components: Option<VersionSuffix>,
+    pub import_library: bool,
     pub rustflags: Vec<String>,
 }
 
+impl LibraryCApiConfig {
+    pub fn sover(&self) -> String {
+        let major = self.version.major;
+        let minor = self.version.minor;
+        let patch = self.version.patch;
+
+        match self.version_suffix_components {
+            None => match (major, minor, patch) {
+                (0, 0, patch) => format!("0.0.{patch}"),
+                (0, minor, _) => format!("0.{minor}"),
+                (major, _, _) => format!("{major}"),
+            },
+            Some(VersionSuffix::Major) => format!("{major}"),
+            Some(VersionSuffix::MajorMinor) => format!("{major}.{minor}"),
+            Some(VersionSuffix::MajorMinorPatch) => format!("{major}.{minor}.{patch}"),
+        }
+    }
+}
+
 #[derive(Debug, Default)]
 pub struct InstallCApiConfig {
     pub include: Vec<InstallTarget>,
@@ -620,6 +647,8 @@ fn load_manifest_capi_config(pkg: &Package) -> anyhow::Result<CApiConfig> {
     let mut version = pkg.version().clone();
     let mut install_subdir = None;
     let mut versioning = true;
+    let mut version_suffix_components = None;
+    let mut import_library = true;
     let mut rustflags = Vec::new();
 
     if let Some(library) = library {
@@ -636,6 +665,23 @@ fn load_manifest_capi_config(pkg: &Package) -> anyhow::Result<CApiConfig> {
             .get("versioning")
             .and_then(|v| v.as_bool())
             .unwrap_or(true);
+
+        if let Some(value) = library.get("version_suffix_components") {
+            let value = value.as_integer().with_context(|| {
+                format!("Value for `version_suffix_components` is not an integer: {value:?}")
+            })?;
+            version_suffix_components = Some(match value {
+                1 => VersionSuffix::Major,
+                2 => VersionSuffix::MajorMinor,
+                3 => VersionSuffix::MajorMinorPatch,
+                _ => anyhow::bail!("Out of range value for version suffix components: {value}"),
+            });
+        }
+
+        import_library = library
+            .get("import_library")
+            .and_then(|v| v.as_bool())
+            .unwrap_or(true);
         if let Some(args) = library.get("rustflags").and_then(|v| v.as_str()) {
             let args = args
                 .split(' ')
@@ -651,6 +697,8 @@ fn load_manifest_capi_config(pkg: &Package) -> anyhow::Result<CApiConfig> {
         version,
         install_subdir,
         versioning,
+        version_suffix_components,
+        import_library,
         rustflags,
     };
 
@@ -992,7 +1040,7 @@ pub fn cbuild(
     default_profile: &str,
 ) -> anyhow::Result<(Vec<CPackage>, CompileOptions)> {
     let rustc = config.load_global_rustc(Some(ws))?;
-    let targets = args.targets();
+    let targets = args.targets()?;
     let target = match targets.len() {
         0 => rustc.host.to_string(),
         1 => targets[0].to_string(),
@@ -1118,7 +1166,7 @@ pub fn cbuild(
 
             build_pc_files(ws, &capi_config.pkg_config.filename, &root_output, &pc)?;
 
-            if !only_staticlib {
+            if !only_staticlib && capi_config.library.import_library {
                 let lib_name = name;
                 build_def_file(ws, lib_name, &rustc_target, &root_output)?;
 
@@ -1241,3 +1289,66 @@ pub fn ctest(
 
     ops::run_tests(ws, &ops, &test_args)
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use semver::Version;
+
+    fn make_test_library_config(version: &str) -> LibraryCApiConfig {
+        LibraryCApiConfig {
+            name: "example".to_string(),
+            version: Version::parse(version).unwrap(),
+            install_subdir: None,
+            versioning: true,
+            version_suffix_components: None,
+            import_library: true,
+            rustflags: vec![],
+        }
+    }
+
+    #[test]
+    pub fn test_semver_zero_zero_zero() {
+        let library = make_test_library_config("0.0.0");
+        let sover = library.sover();
+        assert_eq!(sover, "0.0.0");
+    }
+
+    #[test]
+    pub fn test_semver_zero_one_zero() {
+        let library = make_test_library_config("0.1.0");
+        let sover = library.sover();
+        assert_eq!(sover, "0.1");
+    }
+
+    #[test]
+    pub fn test_semver_one_zero_zero() {
+        let library = make_test_library_config("1.0.0");
+        let sover = library.sover();
+        assert_eq!(sover, "1");
+    }
+
+    #[test]
+    pub fn text_one_fixed_zero_zero_zero() {
+        let mut library = make_test_library_config("0.0.0");
+        library.version_suffix_components = Some(VersionSuffix::Major);
+        let sover = library.sover();
+        assert_eq!(sover, "0");
+    }
+
+    #[test]
+    pub fn text_two_fixed_one_zero_zero() {
+        let mut library = make_test_library_config("1.0.0");
+        library.version_suffix_components = Some(VersionSuffix::MajorMinor);
+        let sover = library.sover();
+        assert_eq!(sover, "1.0");
+    }
+
+    #[test]
+    pub fn text_three_fixed_one_zero_zero() {
+        let mut library = make_test_library_config("1.0.0");
+        library.version_suffix_components = Some(VersionSuffix::MajorMinorPatch);
+        let sover = library.sover();
+        assert_eq!(sover, "1.0.0");
+    }
+}
