@@ -3,7 +3,6 @@ use std::path::{Component, Path, PathBuf};
 
 use cargo::core::Workspace;
 use cargo_util::paths::{copy, create_dir_all};
-use semver::Version;
 
 use crate::build::*;
 use crate::build_targets::BuildTargets;
@@ -90,7 +89,8 @@ impl LibType {
             | ("netbsd", _)
             | ("android", _)
             | ("haiku", _)
-            | ("illumos", _) => LibType::So,
+            | ("illumos", _)
+            | ("emscripten", _) => LibType::So,
             ("macos", _) | ("ios", _) => LibType::Dylib,
             ("windows", _) => LibType::Windows,
             _ => unimplemented!("The target {}-{} is not supported yet", os, env),
@@ -100,43 +100,42 @@ impl LibType {
 
 pub(crate) struct UnixLibNames {
     canonical: String,
-    with_major_ver: String,
+    with_main_ver: String,
     with_full_ver: String,
 }
 
 impl UnixLibNames {
-    pub(crate) fn new(lib_type: LibType, lib_name: &str, lib_version: &Version) -> Option<Self> {
+    pub(crate) fn new(lib_type: LibType, library: &LibraryCApiConfig) -> Option<Self> {
+        let lib_name = &library.name;
+        let lib_version = &library.version;
+        let main_version = library.sover();
+
         match lib_type {
             LibType::So => {
                 let lib = format!("lib{lib_name}.so");
-                let lib_with_major_ver = format!("{}.{}", lib, lib_version.major);
                 let lib_with_full_ver = format!(
-                    "{}.{}.{}",
-                    lib_with_major_ver, lib_version.minor, lib_version.patch
+                    "{}.{}.{}.{}",
+                    lib, lib_version.major, lib_version.minor, lib_version.patch
                 );
+                let lib_with_main_ver = format!("{}.{}", lib, main_version);
+
                 Some(Self {
                     canonical: lib,
-                    with_major_ver: lib_with_major_ver,
+                    with_main_ver: lib_with_main_ver,
                     with_full_ver: lib_with_full_ver,
                 })
             }
             LibType::Dylib => {
                 let lib = format!("lib{lib_name}.dylib");
-                let lib_with_major_ver = if lib_version.major == 0 {
-                    format!(
-                        "lib{}.{}.{}.dylib",
-                        lib_name, lib_version.major, lib_version.minor
-                    )
-                } else {
-                    format!("lib{}.{}.dylib", lib_name, lib_version.major)
-                };
+                let lib_with_main_ver = format!("lib{}.{}.dylib", lib_name, main_version);
+
                 let lib_with_full_ver = format!(
                     "lib{}.{}.{}.{}.dylib",
                     lib_name, lib_version.major, lib_version.minor, lib_version.patch
                 );
                 Some(Self {
                     canonical: lib,
-                    with_major_ver: lib_with_major_ver,
+                    with_main_ver: lib_with_main_ver,
                     with_full_ver: lib_with_full_ver,
                 })
             }
@@ -145,12 +144,14 @@ impl UnixLibNames {
     }
 
     fn links(&self, install_path_lib: &Path) {
-        let mut ln_sf = std::process::Command::new("ln");
-        ln_sf.arg("-sf");
-        ln_sf
-            .arg(&self.with_full_ver)
-            .arg(install_path_lib.join(&self.with_major_ver));
-        let _ = ln_sf.status().unwrap();
+        if self.with_main_ver != self.with_full_ver {
+            let mut ln_sf = std::process::Command::new("ln");
+            ln_sf.arg("-sf");
+            ln_sf
+                .arg(&self.with_full_ver)
+                .arg(install_path_lib.join(&self.with_main_ver));
+            let _ = ln_sf.status().unwrap();
+        }
 
         let mut ln_sf = std::process::Command::new("ln");
         ln_sf.arg("-sf");
@@ -235,12 +236,10 @@ pub fn cinstall(ws: &Workspace, packages: &[CPackage]) -> anyhow::Result<()> {
         if let Some(ref shared_lib) = build_targets.shared_lib {
             ws.config().shell().status("Installing", "shared library")?;
 
-            let lib_name = &capi_config.library.name;
             let lib_type = LibType::from_build_targets(build_targets);
             match lib_type {
                 LibType::So | LibType::Dylib => {
-                    let lib = UnixLibNames::new(lib_type, lib_name, &capi_config.library.version)
-                        .unwrap();
+                    let lib = UnixLibNames::new(lib_type, &capi_config.library).unwrap();
                     lib.install(capi_config, shared_lib, &install_path_lib)?;
                 }
                 LibType::Windows => {
@@ -255,13 +254,14 @@ pub fn cinstall(ws: &Workspace, packages: &[CPackage]) -> anyhow::Result<()> {
                         // We assume they are plugins, install them in the custom libdir path
                         copy(shared_lib, install_path_lib.join(lib_name))?;
                     }
-
-                    let impl_lib = build_targets.impl_lib.as_ref().unwrap();
-                    let impl_lib_name = impl_lib.file_name().unwrap();
-                    copy(impl_lib, install_path_lib.join(impl_lib_name))?;
-                    let def = build_targets.def.as_ref().unwrap();
-                    let def_name = def.file_name().unwrap();
-                    copy(def, install_path_lib.join(def_name))?;
+                    if capi_config.library.import_library {
+                        let impl_lib = build_targets.impl_lib.as_ref().unwrap();
+                        let impl_lib_name = impl_lib.file_name().unwrap();
+                        copy(impl_lib, install_path_lib.join(impl_lib_name))?;
+                        let def = build_targets.def.as_ref().unwrap();
+                        let def_name = def.file_name().unwrap();
+                        copy(def, install_path_lib.join(def_name))?;
+                    }
                 }
             }
         }
